@@ -37,8 +37,10 @@ def Kuf_sqexp_multiscale(inducing_variable: Multiscale, kernel: SquaredExponenti
 
 
 @Kuf.register(InducingPatches, Convolutional, object)
-def Kuf_conv_patch(feat, kern, Xnew):
-    Xp = kern.get_patches(Xnew)  # [N, num_patches, patch_len]
-    bigKzx = kern.base_kernel.K(feat.Z, Xp)  # [M, N, P] -- thanks to broadcasting of kernels
-    Kzx = tf.reduce_sum(bigKzx * kern.weights if hasattr(kern, "weights") else bigKzx, [2])
-    return Kzx / kern.num_patches
+def Kuf_conv_patch(inducing_variable, kernel, Xnew):
+    Xp = kernel.get_patches(Xnew)  # [N, num_patches, patch_len]
+    bigKzx = kernel.base_kernel.K(
+        inducing_variable.Z, Xp
+    )  # [M, N, P] -- thanks to broadcasting of kernels
+    Kzx = tf.reduce_sum(bigKzx * kernel.weights if hasattr(kernel, "weights") else bigKzx, [2])
+    return Kzx / kernel.num_patches
