@@ -16,12 +16,11 @@ import tensorflow as tf
 
 from .. import kernels
 from .. import mean_functions as mfn
-from ..inducing_variables import InducingVariables, InducingPoints
+from ..inducing_variables import InducingPoints, InducingVariables
 from ..probability_distributions import DiagonalGaussian, Gaussian, MarkovGaussian
 from . import dispatch
 from .expectations import expectation
 
-
 NoneType = type(None)
 
 # ================ exKxz transpose and mean function handling =================
