@@ -21,15 +21,19 @@ import tensorflow as tf
 import gpflow
 import gpflow.ci_utils
 from gpflow.conditionals import conditional
+from gpflow.inducing_variables import InducingPoints
+from gpflow.mean_functions import Zero
 from gpflow.models.util import inducingpoint_wrapper
 from gpflow.posteriors import (
     AbstractPosterior,
     FallbackIndependentLatentPosterior,
     FullyCorrelatedPosterior,
+    GPRPosterior,
     IndependentPosteriorMultiOutput,
     IndependentPosteriorSingleOutput,
     LinearCoregionalizationPosterior,
     PrecomputeCacheType,
+    SGPRPosterior,
     create_posterior,
 )
 
@@ -125,7 +129,12 @@ def _register_posterior_test_fixture():
 
 
 def create_conditional(
-    *, kernel, inducing_variable, q_mu, q_sqrt, whiten,
+    *,
+    kernel,
+    inducing_variable,
+    q_mu,
+    q_sqrt,
+    whiten,
 ):
     def conditional_closure(Xnew, *, full_cov, full_output_cov):
         return conditional(
@@ -175,10 +184,18 @@ def test_independent_single_output(
     q_sqrt = q_sqrt_factory(NUM_INDUCING_POINTS, 1)
 
     conditional = create_conditional(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     posterior = create_posterior(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     register_posterior_test(posterior, IndependentPosteriorSingleOutput)
 
@@ -188,7 +205,12 @@ def test_independent_single_output(
 
 
 def test_fully_correlated_multi_output(
-    register_posterior_test, q_sqrt_factory, full_cov, full_output_cov, whiten, output_dims,
+    register_posterior_test,
+    q_sqrt_factory,
+    full_cov,
+    full_output_cov,
+    whiten,
+    output_dims,
 ):
     """
     The fully correlated posterior has one latent GP.
@@ -202,10 +224,18 @@ def test_fully_correlated_multi_output(
     q_sqrt = q_sqrt_factory(output_dims * NUM_INDUCING_POINTS, 1)
 
     conditional = create_conditional(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     posterior = create_posterior(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     register_posterior_test(posterior, FullyCorrelatedPosterior)
 
@@ -237,10 +267,18 @@ def test_independent_multi_output_shk_shi(
     q_sqrt = q_sqrt_factory(NUM_INDUCING_POINTS, num_latent_gps)
 
     conditional = create_conditional(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     posterior = create_posterior(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     register_posterior_test(posterior, IndependentPosteriorMultiOutput)
 
@@ -275,10 +313,18 @@ def test_independent_multi_output_shk_sei(
     q_sqrt = q_sqrt_factory(NUM_INDUCING_POINTS, num_latent_gps)
 
     conditional = create_conditional(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     posterior = create_posterior(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     register_posterior_test(posterior, IndependentPosteriorMultiOutput)
 
@@ -310,10 +356,18 @@ def test_independent_multi_output_sek_shi(
     q_sqrt = q_sqrt_factory(NUM_INDUCING_POINTS, num_latent_gps)
 
     conditional = create_conditional(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     posterior = create_posterior(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     register_posterior_test(posterior, IndependentPosteriorMultiOutput)
 
@@ -348,10 +402,18 @@ def test_independent_multi_output_sek_sei(
     q_sqrt = q_sqrt_factory(NUM_INDUCING_POINTS, num_latent_gps)
 
     conditional = create_conditional(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     posterior = create_posterior(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     register_posterior_test(posterior, IndependentPosteriorMultiOutput)
 
@@ -361,7 +423,12 @@ def test_independent_multi_output_sek_sei(
 
 
 def test_fallback_independent_multi_output_sei(
-    register_posterior_test, q_sqrt_factory, full_cov, full_output_cov, whiten, output_dims,
+    register_posterior_test,
+    q_sqrt_factory,
+    full_cov,
+    full_output_cov,
+    whiten,
+    output_dims,
 ):
     """
     Fallback posterior with separate independent inducing variables.
@@ -380,10 +447,18 @@ def test_fallback_independent_multi_output_sei(
     q_sqrt = q_sqrt_factory(NUM_INDUCING_POINTS, 1)
 
     conditional = create_conditional(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     posterior = create_posterior(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     register_posterior_test(posterior, FallbackIndependentLatentPosterior)
 
@@ -393,7 +468,12 @@ def test_fallback_independent_multi_output_sei(
 
 
 def test_fallback_independent_multi_output_shi(
-    register_posterior_test, q_sqrt_factory, full_cov, full_output_cov, whiten, output_dims,
+    register_posterior_test,
+    q_sqrt_factory,
+    full_cov,
+    full_output_cov,
+    whiten,
+    output_dims,
 ):
     """
     Fallback posterior with shared independent inducing variables.
@@ -412,10 +492,18 @@ def test_fallback_independent_multi_output_shi(
     q_sqrt = q_sqrt_factory(NUM_INDUCING_POINTS, 1)
 
     conditional = create_conditional(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     posterior = create_posterior(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     register_posterior_test(posterior, FallbackIndependentLatentPosterior)
 
@@ -451,10 +539,18 @@ def test_linear_coregionalization_sei(
     q_sqrt = q_sqrt_factory(NUM_INDUCING_POINTS, num_latent_gps)
 
     conditional = create_conditional(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     posterior = create_posterior(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     register_posterior_test(posterior, LinearCoregionalizationPosterior)
 
@@ -487,10 +583,18 @@ def test_linear_coregionalization_shi(
     q_sqrt = q_sqrt_factory(NUM_INDUCING_POINTS, num_latent_gps)
 
     conditional = create_conditional(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     posterior = create_posterior(
-        kernel=kernel, inducing_variable=inducing_variable, q_mu=q_mu, q_sqrt=q_sqrt, whiten=whiten,
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
     )
     register_posterior_test(posterior, LinearCoregionalizationPosterior)
 
@@ -525,6 +629,57 @@ def test_posterior_update_cache_with_variables_no_precompute(
     assert isinstance(posterior.Qinv, tf.Variable)
 
 
+@pytest.mark.parametrize(
+    "precompute_cache_type", [PrecomputeCacheType.NOCACHE, PrecomputeCacheType.TENSOR]
+)
+def test_gpr_posterior_update_cache_with_variables_no_precompute(
+    register_posterior_test, q_sqrt_factory, whiten, precompute_cache_type
+):
+    kernel = gpflow.kernels.SquaredExponential()
+    X = np.random.randn(NUM_INDUCING_POINTS, INPUT_DIMS)
+    Y = np.random.randn(NUM_INDUCING_POINTS, 1)
+
+    posterior = GPRPosterior(
+        kernel=kernel,
+        data=(X, Y),
+        likelihood_variance=gpflow.Parameter(0.1),
+        precompute_cache=precompute_cache_type,
+        mean_function=Zero(),
+    )
+    posterior.update_cache(PrecomputeCacheType.VARIABLE)
+    register_posterior_test(posterior, GPRPosterior)
+
+    assert isinstance(posterior.alpha, tf.Variable)
+    assert isinstance(posterior.Qinv, tf.Variable)
+
+
+@pytest.mark.parametrize(
+    "precompute_cache_type", [PrecomputeCacheType.NOCACHE, PrecomputeCacheType.TENSOR]
+)
+def test_sgpr_posterior_update_cache_with_variables_no_precompute(
+    register_posterior_test, q_sqrt_factory, whiten, precompute_cache_type
+):
+    kernel = gpflow.kernels.SquaredExponential()
+    X = np.random.randn(NUM_INDUCING_POINTS, INPUT_DIMS)
+    Y = np.random.randn(NUM_INDUCING_POINTS, 1)
+    Z = np.random.randn(NUM_INDUCING_POINTS, INPUT_DIMS)
+
+    posterior = SGPRPosterior(
+        kernel=kernel,
+        data=(X, Y),
+        inducing_variable=InducingPoints(Z),
+        likelihood_variance=gpflow.Parameter(0.1),
+        num_latent_gps=1,
+        precompute_cache=precompute_cache_type,
+        mean_function=Zero(),
+    )
+    posterior.update_cache(PrecomputeCacheType.VARIABLE)
+    register_posterior_test(posterior, SGPRPosterior)
+
+    assert isinstance(posterior.alpha, tf.Variable)
+    assert isinstance(posterior.Qinv, tf.Variable)
+
+
 def test_posterior_update_cache_with_variables_update_value(q_sqrt_factory, whiten):
     # setup posterior
     kernel = gpflow.kernels.SquaredExponential()
@@ -558,9 +713,9 @@ def test_posterior_update_cache_with_variables_update_value(q_sqrt_factory, whit
     posterior.update_cache(PrecomputeCacheType.VARIABLE)
 
     # assert that the values have changed
-    assert np.any(np.not_equal(initial_alpha, posterior.alpha))
+    assert not np.allclose(initial_alpha, tf.convert_to_tensor(posterior.alpha))
     if initial_q_sqrt is not None:
-        assert np.any(np.not_equal(initial_Qinv, posterior.Qinv))
+        assert not np.allclose(initial_Qinv, tf.convert_to_tensor(posterior.Qinv))
 
 
 def test_posterior_update_cache_fails_without_argument(q_sqrt_factory, whiten):
@@ -606,3 +761,36 @@ def test_posterior_update_cache_fails_without_argument(q_sqrt_factory, whiten):
     posterior.update_cache()  # does not raise an exception
     assert isinstance(posterior.alpha, tf.Tensor)
     assert isinstance(posterior.Qinv, tf.Tensor)
+
+
+def test_posterior_create_with_variables_update_cache_works(q_sqrt_factory, whiten):
+    # setup posterior
+    kernel = gpflow.kernels.SquaredExponential()
+    inducing_variable = inducingpoint_wrapper(np.random.randn(NUM_INDUCING_POINTS, INPUT_DIMS))
+
+    q_mu = tf.Variable(np.random.randn(NUM_INDUCING_POINTS, 1))
+
+    initial_q_sqrt = q_sqrt_factory(NUM_INDUCING_POINTS, 1)
+    if initial_q_sqrt is not None:
+        q_sqrt = tf.Variable(initial_q_sqrt)
+    else:
+        q_sqrt = initial_q_sqrt
+
+    posterior = IndependentPosteriorSingleOutput(
+        kernel=kernel,
+        inducing_variable=inducing_variable,
+        q_mu=q_mu,
+        q_sqrt=q_sqrt,
+        whiten=whiten,
+        precompute_cache=PrecomputeCacheType.VARIABLE,
+    )
+    assert isinstance(posterior.alpha, tf.Variable)
+    assert isinstance(posterior.Qinv, tf.Variable)
+
+    alpha = posterior.alpha
+    Qinv = posterior.Qinv
+
+    posterior.update_cache()
+
+    assert posterior.alpha is alpha
+    assert posterior.Qinv is Qinv
